import { jsx, jsxs } from "../node_modules/.pnpm/vue-jsx-runtime@0.1.0_vue@3.4.35_typescript@5.5.4_/node_modules/vue-jsx-runtime/dist/vue-jsx-runtime.esm-browser.mjs";
import { getStartTag, getEndTag } from "./utils.mjs";
import merge from "../node_modules/.pnpm/lodash@4.17.21/node_modules/lodash/merge.mjs";
import { unified } from "../node_modules/.pnpm/unified@11.0.5/node_modules/unified/lib/index.mjs";
import remarkParse from "../node_modules/.pnpm/remark-parse@11.0.0/node_modules/remark-parse/lib/index.mjs";
import remarkRehype from "../node_modules/.pnpm/remark-rehype@11.1.0/node_modules/remark-rehype/lib/index.mjs";
import { VFile } from "../node_modules/.pnpm/vfile@6.0.2/node_modules/vfile/lib/index.mjs";
import { visit } from "../node_modules/.pnpm/unist-util-visit@5.0.0/node_modules/unist-util-visit/lib/index.mjs";
import { toJsxRuntime } from "../node_modules/.pnpm/hast-util-to-jsx-runtime@2.3.0/node_modules/hast-util-to-jsx-runtime/lib/index.mjs";
import { urlAttributes } from "../node_modules/.pnpm/html-url-attributes@3.0.0/node_modules/html-url-attributes/lib/index.mjs";
import { Fragment } from "vue";
const emptyPlugins = [];
const emptyRemarkRehypeOptions = {
  allowDangerousHtml: true
};
const safeProtocol = /^(https?|ircs?|mailto|xmpp)$/i;
const matchTag = (tags, value) => {
  if (!Array.isArray(tags)) {
    return false;
  }
  return tags.some(
    (tag) => getStartTag(value, tag) === 0 || getEndTag(value, tag) === 0
  );
};
const markdown_token_raw = "markdown_token_raw";
const getDefaultHandlers = (tags) => {
  return {
    html: (state, node, parent) => {
      if (node.value && matchTag(tags, node.value)) {
        const result = { type: "raw", value: node.value };
        state.patch(node, result);
        return state.applyData(node, result);
      }
      if (state.options.allowDangerousHtml) {
        const result = { type: markdown_token_raw, value: node.value };
        state.patch(node, result);
        return state.applyData(node, result);
      }
    }
  };
};
function Markdown(options) {
  const allowedElements = options.allowedElements;
  const allowElement = options.allowElement;
  const children = options.children || "";
  const className = options.className;
  const components = options.components;
  const disallowedElements = options.disallowedElements;
  const rehypePlugins = options.rehypePlugins || emptyPlugins;
  const remarkPlugins = options.remarkPlugins || emptyPlugins;
  let handlers = {};
  if (Array.isArray(options.customElements) && options.customElements.length > 0) {
    handlers = getDefaultHandlers(options.customElements);
  }
  const _remarkRehypeOptions = merge({ handlers }, options.remarkRehypeOptions);
  const remarkRehypeOptions = options.remarkRehypeOptions ? { ..._remarkRehypeOptions, ...emptyRemarkRehypeOptions } : { ...emptyRemarkRehypeOptions, handlers };
  const skipHtml = options.skipHtml;
  const unwrapDisallowed = options.unwrapDisallowed;
  const urlTransform = options.urlTransform || defaultUrlTransform;
  const processor = unified().use(remarkParse).use(remarkPlugins).use(remarkRehype, remarkRehypeOptions).use(rehypePlugins);
  const file = new VFile();
  if (typeof children === "string") {
    file.value = children;
  }
  const mdastTree = processor.parse(file);
  let hastTree = processor.runSync(mdastTree, file);
  if (className) {
    hastTree = {
      type: "element",
      tagName: "div",
      properties: { className },
      children: hastTree.type === "root" ? hastTree.children : [hastTree]
    };
  }
  visit(hastTree, transform);
  const tree = toJsxRuntime(hastTree, {
    Fragment,
    components,
    ignoreInvalidStyle: true,
    jsx,
    jsxs,
    passKeys: true,
    passNode: true,
    elementAttributeNameCase: "html"
  });
  return tree;
  function transform(node, index, parent) {
    if (node.type === "raw" && parent && typeof index === "number") {
      if (skipHtml) {
        parent.children.splice(index, 1);
      } else {
        parent.children[index] = { type: "text", value: node.value };
      }
      return index;
    }
    if (node.type === markdown_token_raw && parent && typeof index === "number") {
      console.log("markdown_token_raw——transform", node, index, parent);
      if (skipHtml) {
        parent.children.splice(index, 1);
      } else {
        parent.children[index] = { type: "text", value: node.value };
      }
      return index;
    }
    if (node.type === "element") {
      let key;
      for (key in urlAttributes) {
        if (urlAttributes.hasOwnProperty(key) && node.properties.hasOwnProperty(key)) {
          const value = node.properties[key];
          const test = urlAttributes[key];
          if (test === null || test.includes(node.tagName)) {
            node.properties[key] = urlTransform(String(value || ""), key, node);
          }
        }
      }
    }
    if (node.type === "element") {
      let remove = allowedElements ? !allowedElements.includes(node.tagName) : disallowedElements ? disallowedElements.includes(node.tagName) : false;
      if (!remove && allowElement && typeof index === "number") {
        remove = !allowElement(node, index, parent);
      }
      if (remove && parent && typeof index === "number") {
        if (unwrapDisallowed && node.children) {
          parent.children.splice(index, 1, ...node.children);
        } else {
          parent.children.splice(index, 1);
        }
        return index;
      }
    }
  }
}
function defaultUrlTransform(value) {
  const colon = value.indexOf(":");
  const questionMark = value.indexOf("?");
  const numberSign = value.indexOf("#");
  const slash = value.indexOf("/");
  if (
    // If there is no protocol, it’s relative.
    colon < 0 || // If the first colon is after a `?`, `#`, or `/`, it’s not a protocol.
    slash > -1 && colon > slash || questionMark > -1 && colon > questionMark || numberSign > -1 && colon > numberSign || // It is a protocol, it should be allowed.
    safeProtocol.test(value.slice(0, colon))
  ) {
    return value;
  }
  return "";
}
export {
  Markdown,
  defaultUrlTransform,
  getDefaultHandlers,
  markdown_token_raw
};
