"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const vue = require("vue");
const highlight = require("./highlight.js");
const refractor = require("./refractor.js");
const CodeHighLight = /* @__PURE__ */ vue.defineComponent({
  name: "HighlighterVue",
  props: {
    generatorType: {
      type: String,
      default: "highlight"
    },
    code: {
      type: String,
      required: true
    },
    language: {
      type: String,
      default: ""
    },
    autoMatch: {
      type: Boolean,
      default: true
    },
    /**generatorType === refractor => enable  */
    theme: {
      type: Object,
      default: () => ({})
    },
    //--------------------------------------------------------
    /* generatorType === highlight => enable */
    ignoreIllegals: {
      type: Boolean,
      default: true
    }
  },
  setup(props) {
    const language = vue.ref(props.language);
    vue.watch(() => props.language, (newLanguage) => {
      language.value = newLanguage;
    });
    const autodetect = vue.computed(() => props.autoMatch || !language.value);
    const cannotDetectLanguage = vue.computed(() => !autodetect.value && !highlight.hljsGetLanguage(language.value));
    const className = vue.computed(() => {
      if (cannotDetectLanguage.value) {
        return "";
      } else {
        return `hljs ${language.value}`;
      }
    });
    const highlightedCode = vue.computed(() => {
      if (props.generatorType === "highlight") {
        return highlight.highlightTohtml(props.code, {
          autoMatch: props.autoMatch,
          language: props.language
        });
      } else {
        return refractor.refractorToHtml(props.code, {
          theme: props.theme
        });
      }
    });
    return {
      className,
      highlightedCode
    };
  },
  render() {
    return vue.h("pre", {}, [vue.h("code", {
      class: this.className,
      innerHTML: this.highlightedCode
    })]);
  }
});
exports.CodeHighLight = CodeHighLight;
