"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const vue = require("vue");
/*!
  * vue-jsx-runtime v0.1.0
  * (c) 2021 dolymood (dolymood@gmail.com)
  * @license MIT
  */
const hasOwnProperty = Object.prototype.hasOwnProperty;
const hasOwn = (val, key) => hasOwnProperty.call(val, key);
const isFunction = (val) => typeof val === "function";
const isString = (val) => typeof val === "string";
const isSymbol = (val) => typeof val === "symbol";
const isObject = (val) => val !== null && typeof val === "object";
const isArray = Array.isArray;
const vModelEleDirTypes = {
  select: vue.vModelSelect,
  textarea: vue.vModelText
};
const vModelDirTypes = {
  default: vue.vModelText,
  radio: vue.vModelRadio,
  checkbox: vue.vModelCheckbox,
  dynamic: vue.vModelDynamic
};
const DIR_CASES = {
  // directives
  html: (type, val, props) => {
    props.innerHTML = val;
  },
  text: (type, val, props) => {
    props.textContent = vue.toDisplayString(val);
  },
  show: (type, val, props, directives) => {
    directives.push([vue.vShow, val]);
  },
  model: (type, val, props, directives, config, arg) => {
    const isPlainNode = isString(type) || isSymbol(type);
    const [{ get, set }, args, modifiers] = processDirVal(val, arg);
    const prop = args || "modelValue";
    props[prop] = get();
    const fixProp = prop === "modelValue" ? "model" : prop;
    const fixModifiersKey = `${fixProp}Modifiers`;
    props[fixModifiersKey] = modifiers;
    props[`onUpdate:${prop}`] = set;
    const eleDir = vModelEleDirTypes[type];
    const typeDir = isString(config.type) || !config.type ? vModelDirTypes[config.type] || vModelDirTypes.default : vModelDirTypes.dynamic;
    const directiveName = eleDir || typeDir;
    directives.push([
      directiveName,
      props[prop],
      isPlainNode ? void 0 : prop,
      props[fixModifiersKey]
    ]);
    if (isPlainNode) {
      delete props[prop];
      delete props[fixModifiersKey];
    } else {
      directives.pop();
    }
  },
  slots: (type, val, props, directives, config) => {
    PROP_CASES.children(type, config.children, props, directives, config);
    const children = props.children;
    if (!children || !children.default) {
      let defaultSlot = children;
      if (children) {
        if (!isFunction(children)) {
          defaultSlot = () => children;
        }
      }
      props.children = {};
      if (defaultSlot) {
        props.children.default = defaultSlot;
      }
    }
    Object.assign(props.children, val);
  },
  __custom__: (type, val, props, directives, config, arg, name) => {
    const [{ get, set }, args, modifiers] = processDirVal(val, arg);
    props[`onUpdate:${name}`] = set;
    directives.push([
      vue.resolveDirective(name),
      get(),
      args,
      modifiers
    ]);
  }
};
const PROP_CASES = {
  children: (type, nodes, props) => {
    if (props.__processed__children)
      return;
    props.__processed__children = true;
    if (isString(type) || isSymbol(type)) {
      if (!isArray(nodes) && vue.isVNode(nodes)) {
        nodes = [nodes];
      }
      props.children = nodes;
    } else {
      if (vue.isVNode(nodes) || isArray(nodes) || !isObject(nodes)) {
        props.children = {
          default: isFunction(nodes) ? nodes : () => nodes
        };
      } else {
        props.children = nodes;
      }
    }
  }
};
const reversedProps = {
  children: 1,
  __processed__children: 1
};
const xlinkRE = /^xlink([A-Z])/;
function jsx$1(type, config = {}, maybeKey, source, self) {
  let propName;
  const directives = [];
  const props = {};
  for (propName in config) {
    if (hasOwn(config, propName)) {
      propName = propName;
      const val = config[propName];
      if (/^(v-[A-Za-z0-9-])/.test(propName)) {
        const match = /(?:^v-([a-z0-9-]+))?(?:(?::)([^\.]+))?(.+)?$/i.exec(propName);
        const dirName = match[1];
        const arg = match[2];
        const dirFn = DIR_CASES[dirName] || DIR_CASES.__custom__;
        dirFn(type, val, props, directives, config, arg, dirName);
      } else {
        if (hasOwn(PROP_CASES, propName)) {
          const propVal = PROP_CASES[propName](type, val, props, directives, config);
          if (propVal !== void 0) {
            props[propName] = propVal;
          }
        } else {
          if (propName.match(xlinkRE)) {
            propName = propName.replace(xlinkRE, (_, firstCharacter) => `xlink:${firstCharacter.toLowerCase()}`);
          }
          props[propName] = val;
        }
      }
    }
  }
  const children = props.children;
  Object.keys(reversedProps).forEach((k) => {
    delete props[k];
  });
  if (maybeKey !== void 0 && !hasOwn(props, "key")) {
    props.key = "" + maybeKey;
  }
  const vnode = vue.createVNode(type, props, children);
  if (directives.length) {
    return vue.withDirectives(vnode, directives);
  }
  return vnode;
}
const jsxs$1 = jsx$1;
function processDirVal(val, arg) {
  if (!isArray(val)) {
    val = [val, "value"];
  }
  let [value, path, args, modifiers] = val;
  if (isArray(args)) {
    modifiers = args;
    args = void 0;
  }
  if (!arg) {
    arg = args;
  }
  const _modifiers = modifiers ? processModifiers(modifiers) : void 0;
  const paths = isArray(path) ? path : [path];
  const get = () => {
    return baseGetSet(value, paths);
  };
  const set = (v) => {
    return baseGetSet(value, paths, v);
  };
  return [{ get, set }, arg, _modifiers];
}
function processModifiers(modifiers) {
  modifiers = Array.isArray(modifiers) ? modifiers : modifiers ? [modifiers] : [];
  const realModifiers = {};
  modifiers.forEach((modifier) => {
    realModifiers[modifier] = true;
  });
  return realModifiers;
}
function baseGetSet(object, paths, set) {
  let val = object;
  let index = 0;
  const length = paths.length;
  const max = length - 1;
  while (val != null && index <= max) {
    if (set && index === max) {
      val[paths[index]] = set;
      return set;
    }
    val = val[paths[index++]];
  }
  return index && index == length ? val : void 0;
}
function jsxDEV$1(type, config = {}, maybeKey, source, self) {
  if (process.env.NODE_ENV !== "production") {
    return jsx$1(type, config, maybeKey);
  }
}
function jsxWithValidation(type, props = {}, key, isStaticChildren, source, self) {
  return jsxDEV$1(type, props, key);
}
function jsxWithValidationStatic(type, props = {}, key) {
  if (process.env.NODE_ENV !== "production") {
    return jsxWithValidation(type, props, key);
  }
}
function jsxWithValidationDynamic(type, props = {}, key) {
  if (process.env.NODE_ENV !== "production") {
    return jsxWithValidation(type, props, key);
  }
}
const jsx = process.env.NODE_ENV !== "production" ? jsxWithValidationDynamic : jsx$1;
const jsxs = process.env.NODE_ENV !== "production" ? jsxWithValidationStatic : jsxs$1;
process.env.NODE_ENV !== "production" ? jsxWithValidation : void 0;
Object.defineProperty(exports, "Fragment", {
  enumerable: true,
  get: () => vue.Fragment
});
exports.jsx = jsx;
exports.jsxs = jsxs;
